%%%MATLAB code for paper Combinatorial Network Optimization with Unknown Variables: Multi-Armed Bandits with Linear Rewards and Individual Observations
%%%IEEE/ACM Transactions on Networking, vol. 20, no. 5, 2012.
%%%(Code may be used freely, with a suitable acknowledgment: 
%%%This code was obtained from research conducted by the University of Southern California's 
%%%Autonomous Networks Research Group, http://anrg.usc.edu".)
%%%Upload to ANRG.USC.EDU (JUNE 2012) by Yi Gai

clear all;

N = 9;
M = 5;
mtx = [0.3, 0.5, 0.9, 0.7, 0.8, 0.9, 0.6, 0.8, 0.7; 
       0.2, 0.2, 0.3, 0.4, 0.5, 0.4, 0.5, 0.6, 0.9;
       0.8, 0.6, 0.5, 0.4, 0.7, 0.2, 0.8, 0.2, 0.8;
       0.9, 0.2, 0.2, 0.8, 0.3, 0.9, 0.6, 0.5, 0.4;
       0.6, 0.7, 0.5, 0.7, 0.6, 0.8, 0.2, 0.6, 0.8]
total_no = factorial(N)/factorial(N-M)
rnd_end =  2000;
mul = 1000;


[assign,bestValue] = munkres(-mtx)

bestValue = -bestValue;
time_end =  rnd_end*mul+total_no;



full_perm = zeros(total_no,M);
len_comb = factorial(M);

combos = combntns(1:N,M);
len_combos = length(combos);

for i = 1:len_combos
    tmp_array = perms(combos(i,:));
    for j = 1:len_comb
        full_perm((i-1)*len_comb+j,:) = tmp_array(j,:);
    end
end

%%%%%%%declare%%%%
tt = zeros(1,total_no);
nn = ones(1,total_no);
curr_x = zeros(M,N);
arm = zeros(1,M);
reward = 0;
regret = 0;
reg_naive = zeros(1,(rnd_end+total_no-1));
plot_time_naive = zeros(1,(rnd_end+total_no-1));
cnt = 0;
%%%%%%Init%%%%%%%%%%
full_perm;
%loop for all arm
    [curr_x] = currentValue(mtx,M,N);
    arm = full_perm(1,:);
    for i = 1:M
        tt(1) = tt(1)+ curr_x(i,arm(i));
    end
    reward = reward + tt(1);
    regret = bestValue*1 - reward;
for k=2:total_no
    [curr_x] = currentValue(mtx,M,N);
    arm = full_perm(k,:);
    for i = 1:M
        tt(k) = tt(k)+ curr_x(i,arm(i));
    end
    reward = reward + tt(k);
    regret = bestValue*k - reward;
    %plot(k,regret/log(k),'-');
    cnt = cnt + 1;
    reg_naive(cnt) = regret/log(k);
    plot_time_naive(cnt) = k;
end

%plot(1:total_no,regret(1:total_no))

%pre_regret = regret;
%%%%%%Main_Loop%%%%%%%%%%

t = total_no;

for j = 1:rnd_end
    for h = 1:mul
        t = t+1;
        [curr_x] = currentValue(mtx,M,N);
        withConf = tt./nn+sqrt(2*log(t)./nn);
        [max_temp,I] = max(withConf);
        %%%chosse I-th arm%%%
        arm = full_perm(I,:);
        for i = 1:M
            reward = reward+ curr_x(i,arm(i));
            tt(I) = tt(I)+ curr_x(i,arm(i));
        end
        nn(I) = nn(I)+1;
    end
    regret = bestValue*t - reward;
    %plot([t-mul,t],[(pre_regret)/log(t-mul),regret/log(t)],'-');
    %pre_regret = regret;
    cnt = cnt + 1;
    reg_naive(cnt) = regret/log(t);
    plot_time_naive(cnt) = t;    
end

%%%%%%%declare%%%%
tt = zeros(M,N);
nn = zeros(M,N);
curr_x = zeros(M,N);
arm = zeros(1,M);
reward = 0;
regret = 0;
t = 0;
reg_my = zeros(1,(rnd_end+M*N-1));
plot_time_my = zeros(1,(rnd_end+M*N-1));
cnt = 0;
%%%%%%Init%%%%%%%%%%
full_perm;
%loop for all arm
for i=1:M
    for j=1:N
        [curr_x] = currentValue(mtx,M,N);
        for k=1:total_no
            if full_perm(k,i)==j
                t = t + 1;
                for h = 1:M
                    yindex = full_perm(k,h);
                    reward = reward + curr_x(h,yindex);
                    tt(h,yindex) = tt(h,yindex) + curr_x(h,yindex);
                    nn(h,yindex) = nn(h,yindex) + 1;
                end
                if t>1
                    regret = bestValue*t - reward;
                    %plot(t,regret/log(t),'k-');
                    cnt = cnt + 1;
                    reg_my(cnt) = regret/log(t);
                    plot_time_my(cnt) = t;    
                end
                break;
            end
        end
    end
end
%%%%%%Main_Loop%%%%%%%%%%
%pre_regret = regret;
for j = 1:rnd_end
    for h = 1:mul
        t = t+1;
        [curr_x] = currentValue(mtx,M,N);
        matrixConf = tt./nn+sqrt(2*log(t)./nn);
        [arm,currBest] = munkres(-matrixConf);
        
        %%%chosse arm arm%%%
        for i = 1:M
            yindex = arm(i);
            reward = reward+ curr_x(i,yindex);
            tt(i,yindex) = tt(i,yindex) + curr_x(i,yindex);
            nn(i,yindex) = nn(i,yindex) + 1;
        end
    end
    regret = bestValue*t - reward;
    %plot([t-mul,t],[(pre_regret)/log(t-mul),regret/log(t)],'k-');
    %pre_regret = regret;
    cnt = cnt + 1;
    reg_my(cnt) = regret/log(t);
    plot_time_my(cnt) = t;    
end

hold on;
plot(plot_time_naive,reg_naive);
plot(plot_time_my,reg_my,'k');
%axis([0 1e6 0 700]);
xlabel('Time');
ylabel('Regret/Log(t)');
legend('Naive Policy','LLR Policy', 1);
axis([0 2e6 0 3e4]);

